/* $Id: main.c,v 1.14 1998/02/11 04:46:26 svmcguir Exp $ */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "path.h"
#include "option.h"
#include "mesh.h"
#include "util.h"
#include "inner.h"

#define PHRASE "smuggle munitions KGB NORAD Albanian Serbian Ortega cracking radar"

void innerControlVariate(int whichInnerControl,
			 meshData *mesh_in, meshData *mesh_out);

void sumWeights(meshData *md);

double  OuterCtlEuroMaxMeshValue(meshData *md);

double nuIndEst(double S, double r, double dividend, double sigma,
		double T, double t, double strikePrice);

void option_error(char *argv[]){
  printf("Usage: %s parameters\n",argv[0]);
  printf("\t-n underliers (n from Broadie and Glasserman)\n");
  printf("\t-r risk-free rate (r)\n");
  printf("\t-D dividend rate (\\delta)\n");
  printf("\t-v variance (\\sigmna^2)\n");
  printf("\t-T time to maturity (T)\n");
  printf("\t-K strikeprice (K)\n");
  printf("\t-S initial price (S_0)\n");
  printf("\t-N repetitions (N)\n");
  printf("\t-b paths in mesh (b)\n");
  printf("\t-p paths in path estimate (n_p)\n");
  printf("\t-d time slices (d)\n");
  printf("\nRates and variance are for T=1.\n");
  printf("All parameters are required.\n\n");

  exit(1);
}

int main(int argc, char *argv[]){
  meshData md;
  int i,j,time, path, underlier;
  double europe_value, ma,mv,pa,pv,cma,cmv,cpa,cpv,me,pe; 
  int reps;

  const char *options = "n:r:D:v:T:K:S:N:b:p:d:";
  int n,N,b,p,d,option,count=0;
  double T, r,D,v,K,S;
	double euromax, euromax_avg, euromax2_avg, euromax_var, euromax_vol;


  /* Read command line parameters */
  while((option = getopt(argc,argv,options)) != EOF){
    switch(option){
    case('n'):
      sscanf(optarg,"%i",&n);
      count++;
      break;
    case('r'):
      sscanf(optarg,"%lf",&r);
      count++;
      break;
    case('D'):
      sscanf(optarg,"%lf",&D);
      count++;
      break;
    case('v'):
      sscanf(optarg,"%lf",&v);
      count++;
      break;
    case('T'):
      sscanf(optarg,"%lf",&T);
      count++;
      break;
    case('K'):
      sscanf(optarg,"%lf",&K);
      count++;
      break;
    case('S'):
      sscanf(optarg,"%lf",&S);
      count++;
      break;
    case('N'):
      sscanf(optarg,"%i",&N);
      count++;
      break;
    case('b'):
      sscanf(optarg,"%i",&b);
      count++;
      break;
    case('p'):
      sscanf(optarg,"%i",&p);
      count++;
      break;
    case('d'):
      sscanf(optarg,"%i",&d);
      count++;
      break;
    default:
      option_error(argv);
    }
  }

  printf("%i\n",count);
  if(count != 11){
    option_error(argv);
  }

  printf("n = %i\n",n);
  printf("T = %lf\n",T);
  printf("N = %i\n",N);
  printf("b = %i\n",b);
  printf("p = %i\n",p);
  printf("d = %i\n",d);
  printf("r = %lf\n",r);
  printf("D = %lf\n",D);
  printf("v = %lf\n",v);
  printf("K = %lf\n",K);
  printf("S = %lf\n",S);
  
  /* The parameters that can be set before memory allocation */
  /* times, paths and underliers required to come before allocation. */
  /* Later, these should go in a config file. */
  md.option.pData.times = d + 1;
  md.option.pData.paths = b + p;
  md.option.pData.underliers = n;
  md.option.strikePrice = K;
  md.option.riskFreeRate = r*T/(double)d;
  md.meshPaths = b;
  strcpy(md.option.pData.phrase,PHRASE);

	printf("Got here 1 \n");
  /* Allocate the memory */
  pathDataAllocate(&(md.option.pData));
  optionDataAllocate(&(md.option));
  meshDataAllocate(&md);

  /* Fill the covariance matrix and mean vector */
  /* In our case, the covariance matrix is diagonal with the variance */
  /* in all diagonal entries. */
  for(i=0;i<md.option.pData.underliers;i++){
    for(j=0;j<md.option.pData.underliers;j++){
      if(i == j){
	md.option.pData.covariance[i][j] = v*(double)T/(double)d;
	md.option.pData.growthRate[i] = r*(double)T/(double)d;
	md.option.pData.dividendRate[i] = D*(double)T/(double)d;
      }else{
	md.option.pData.covariance[i][j] = 0;
      }
    }
  }

	printf("Got here 2 \n");

  /* Sets up internal RNG stuff */
  /* Must come after the means and covariances */
  pathParamsInit(&(md.option.pData));

  /* Set all the initial prices */
  for(path=0; path<md.option.pData.paths; path++){
    for(underlier=0; underlier < md.option.pData.underliers; underlier++){
      md.option.pData.price[0][path][underlier] = S;
    }
  }

	printf("Got here 3 \n");

  ma = 0; 
  mv = 0; 
  pa = 0; 
  pv = 0; 
  cma = 0; 
  cmv = 0; 
  cpa = 0; 
  cpv = 0; 
	euromax_avg = euromax2_avg = 0.0;
	printf("Got here 3 \n");
	printf("repetitions=%i \n",N);
  for(reps=1;reps<=N;reps++){
		pathDataFill(&(md.option.pData));
		optionDataFill(&(md.option));
		meshSetFinalValue(&md);
		meshDataFill(&md); 

/*     me = md.valueIfHeld[0][0]; 
     ma += me; 
     mv += me*me; 
     pe = meshPathEstimate(&md); 
     pa += pe; 
     pv += pe*pe; 
     printf("%lf\t",me); 
     printf("%lf\t",pe);  

    innerDataFill(&md);

     me = md.newValueIfHeld[0][0];
     cma += me;
     cmv += me*me;
     pe = innerControlledPathEstimate(&md);
     cpa += pe;
     cpv += pe*pe; */
/*     printf("%lf\t",me);
     printf("%lf\n",pe); */

    /*   innerControlVariate(1, &md, &md); */
    /*   printf("Mesh estimate after inner control= %lf\n",md.valueIfHeld[0][0]); */
    /*   printf("Path estimate After = %lf\n",meshPathEstimate(&md)); */

/*********** Test the outer control european maximum option value ********/
	   printf("MAIN>>> in Outer reps=%i \n",reps);
		euromax = OuterCtlEuroMaxMeshValue(&md);
	   printf("MAIN>>> out Outer reps=%i \n",reps);
	   printf("MAIN>>> out euromax=%lf \n",euromax);
		euromax_avg += euromax;
		euromax2_avg += euromax*euromax;
		printf("MAIN>>> out euromax_avg=%lf \n",euromax_avg);
		printf("MAIN>>> out euromax2_avg=%lf \n",euromax2_avg);
      printf("Iteration: %i Option value:%lf\n",reps,euromax);
  }

	printf("Got here 4 \n");
   reps--;
   ma /= reps;
   pa /= reps;
   cma /= reps;
   cpa /= reps;
   mv /= reps;
   pv /= reps;
   cmv /= reps;
   cpv /= reps;
   printf("reps = %i\n",reps);
/*   printf("%lf\t%lf\t%lf\t%lf\n",ma,pa,cma,cpa);
   printf("%lf\t%lf\t%lf\t%lf\n",mv-ma*ma,pv-pa*pa,cmv-cma*cma,cpv-cpa*cpa);
   printf("%lf\t%lf\t%lf\t%lf\n",sqrt(mv-ma*ma),
 	 sqrt(pv-pa*pa),
 	 sqrt(cmv-cma*cma),
	 sqrt(cpv-cpa*cpa)); 
   printf("%lf\t%lf\t%lf\t%lf\n",sqrt((mv-ma*ma)/reps),
 	 sqrt((pv-pa*pa)/reps),
 	 sqrt((cmv-cma*cma)/reps),
	 sqrt((cpv-cpa*cpa)/reps));  */

/*********** Test the outer control european maximum option value ********/
		euromax_avg /= N;
		euromax_var = euromax2_avg/N - euromax_avg*euromax_avg;
		euromax_vol = sqrt(euromax_var);
   printf("European option average:     %lf\n",euromax_avg);
   printf("European option variability: %lf\n",euromax_var);
   printf("European option volatility:  %lf\n",euromax_vol);
}

void sumWeights(meshData *meshData){
  int t, i, j;
  double sum;
  FILE *fp;

  fp = fopen("wghtavgs.dat", "w+");

  for(t=0; t<meshData->option.pData.times; t++){
    sum = 0.0;
    for(i=0; i < meshData->option.pData.paths; i++){
      for(j=0; j < meshData->meshPaths; j++){
        sum += meshData->weights[t][i][j];
      }
    }
    fprintf(fp,"t = %d  sum = %lf \n",t,sum); 
  }
  fclose(fp);

}
